try:
    from cozmo_fsm import *
except:
    raise ImportError("Could not find cozmo_fsm.")

import math

def calcDist(pose1, pose2):
    xdiff = pose1.position.x - pose2.position.x
    ydiff = pose2.position.y - pose2.position.y
    return math.sqrt(xdiff*xdiff + ydiff*ydiff)

def calcAngle(cube):
    cubeX = cube.pose.position.x
    cubeY = cube.pose.position.y
    robotX = robot.pose.position.x
    robotY = robot.pose.position.y
    Ydiff = cubeY - robotY
    Xdiff = cubeX - robotX
    bearing = atan2(Ydiff, Xdiff)
    turnAngle = radians(bearing) - robot.pose_angle
    return turnAngle

class FindClosestCube(StateNode):
    def start(self, event=None):
        dist1 = calcDist(robot.pose, cube1.pose)
        dist2 = calcDist(robot.pose, cube2.pose)
        dist3 = calcDist(robot.pose, cube3.pose)
        lowest = dist1
        closestCube = cube1
        if (dist2 < lowest):
            lowest = dist2
            closestCube = cube2
        if (dist3 < lowest):
            lowest = dist3
            closestCube = cube3
        
        super().start(event)
        self.post_data(closestCube)

class TurnToClosest(Turn):
    def start(self, event=None):
        if isinstance(event, DataEvent):
            cube = event.data
            self.angle = calcAngle(cube)
        super().start(event)

class Lab3P1(StateMachineProgram):
    def setup(self):
        """
            FindClosestCube() =D=> TurnToClosest()
        """
        
        # Code generated by genfsm on Thu Feb  6 21:11:28 2020:
        
        findclosestcube1 = FindClosestCube() .set_name("findclosestcube1") .set_parent(self)
        turntoclosest1 = TurnToClosest() .set_name("turntoclosest1") .set_parent(self)
        
        datatrans1 = DataTrans() .set_name("datatrans1")
        datatrans1 .add_sources(findclosestcube1) .add_destinations(turntoclosest1)
        
        return self

